\name{compare}
\alias{compare}
\title{Interactively show trace, ACF, and histogram for WinBUGS output.}
\description{
  Given a set of rube() objects, interactively compare their posteriors
  showing three key forms of the  posterior (trace, ACF, and histogram), 
  allowing interactively choosing parameters to see.  For vector and array parameters
  the first view is a set of posterior intervals, maximum lags above
  noise level, and Gelman RHat values.  The set is a random sample
  if the number of elements is greater than nSample.  From this
  overview, you can use previous/next to step through the detailed
  plots for each element.
}
\usage{
compare(..., params = NULL, chains = rep(1, length(MCMCRslts)), 
    nSample = 20, true = NULL, PIprob = 0.95, rHatGrayZone = c(1.2, 2),
    noMenu = FALSE, idString=NULL, cex.main=1)}
\arguments{
  \item{...}{Two or more rube() (or bugs()) objects from successful WinBUGS runs.}
  \item{MCMCRslts}{A list of rube() objects.}
  \item{params}{An optional string vector containing only the parameters of
   interest.}
  \item{chains}{A vector choosing which chains to use in the comparisons.}
  \item{nSample}{Maximum number of elements of a vector parameter to show at once.}
  \item{true}{A true parameter value to mark on the histogram.}
  \item{PIprob}{A value for the posterior credible interval or NULL for none.}
  \item{rHatGrayZone}{Gray zone for color flagging of Gelman's Rhat.}
  \item{noMenu}{Set to TRUE to make one plot without allocating space for a menu.}
  \item{idString}{Text identifying models; shown in outer title.}
  \item{cex.main}{Scale factor for outer title.}
}
\details{
  This is interactive in terms of examining multiple parameters by clicking on the plot.
  The instructions are in the plot's menu when active; R commands can only
  be entered after clicking Quit.
}
\value{
Returns null invisibly.
}
\references{http://www.stat.cmu.edu/~hseltman/rube/}
\author{Howard J. Seltman <hseltman@stat.cmu.edu>}
\seealso{
\code{\link{rube}}
}
\keyword{WinBUGS}
\keyword{posterior}
