\name{p3}
\alias{p3}
\title{Interactively show trace, ACF, and histogram for WinBUGS output.}
\description{
  Given an R2WinBUGS bugs() object or a sims.array component of that, 
  interactively show three key forms of the posterior (trace,
  ACF, and histogram), allowing interactively choosing the parameters.
  For vector and array parameters
  the first view is a set of posterior intervals, maximum lags above
  noise level, and Gelman RHat values.  The set is a random sample
  if the number of elements is greater than nSample.  From this
  overview, you can use previous/next to step through the detailed
  plots for each element.
}
\usage{
p3(rubeResult, params = NULL,  only=NULL, drop=0, add=NULL,
   nSample=20, true=NULL, viewGroups=NULL,
   LCzero=TRUE, PIprob=0.95, rHatGrayZone=c(1.2,2.0), noMenu=FALSE)}
\arguments{
  \item{rubeResult}{A bugs() object of the R2WinBUGS package or its 
   sims.array component.}
  \item{params}{An optional string vector containing only the parameters of
   interest.}
  \item{only}{If "params" specifies a single multi-level parameter, then "only"
   specifies which component of that parameter to display.}
  \item{drop}{Number of initial iterations to drop}
  \item{add}{List of expressions or strings defining new parameters to create.}
  \item{nSample}{Sample size for condensed plot when parameter dim > nSample.}
  \item{true}{An optional numeric vector of true values (for simulated data) with
   NAs as place holders for those without known true values.}
  \item{viewGroups}{A list of string vectors of parameter names to be shown
   in groups.  Any group can be a single string of the form "bb*cc" in which
   case all paramteters starting with "bb" and ending with "cc" will be in the group.}
  \item{LCzero}{Set to FALSE to suppress the zero line in the summary plots for LC groups.}
  \item{PIprob}{A value for the posterior credible interval or NULL for none.}
  \item{rHatGrayZone}{Gray zone for color flagging of Gelman's Rhat.}
  \item{noMenu}{Set to TRUE to make one plot without allocating space for a menu.}
}
\details{
  This is interactive in terms of examining multiple parameters by clicking on the plot.
  The instructions are in the plot's menu when active; R commands can only
  be entered after clicking Quit to end the function.
}
\value{
Returns null invisibly.
}
\references{http://www.stat.cmu.edu/~hseltman/rube/}
\author{Howard J. Seltman <hseltman@stat.cmu.edu>}
\seealso{
\code{\link{rube}}
}
\examples{
cat("Need a good example!\n")
}
\keyword{WinBUGS}
\keyword{posterior}
